﻿using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;
using System;
using System.Linq;

namespace ParcelFabricSDK
{
  internal class Merge2 : Button
  {
    protected async override void OnClick()
    {
      string sTargetParcelType = Microsoft.VisualBasic.Interaction.InputBox("Target Parcel Type:",
        "Merge To Parcel Type", "Tax");
      if (sTargetParcelType.Trim().Length == 0)
        return;
      //jump to the cim thread
      string errorMessage = await QueuedTask.Run( async () =>
      {
        // check for selected layer
        if (MapView.Active.GetSelectedLayers().Count == 0)
          return "Please select a feature layer in the table of contents";

        #region Get Parcel Fabric Stuff
        //first get the feature layer that's selected in the table of contents
        var featSrcLyr = MapView.Active.GetSelectedLayers().OfType<FeatureLayer>().FirstOrDefault();
        if (featSrcLyr.SelectionCount == 0)
          return "There is no selection on the source layer.";
        var myParcelFabricLayer = 
          MapView.Active.Map.GetLayersAsFlattenedList().OfType<ParcelLayer>().FirstOrDefault();
        if (myParcelFabricLayer == null)
          return "Please add a parcel layer to the map.";
        #endregion
        
        try
        {
          #region Edit Operation
          var typeNamesEnum = await myParcelFabricLayer.GetParcelPolygonLayerByTypeName(sTargetParcelType);
          if (typeNamesEnum.Count() == 0)
            return "Target parcel type " + sTargetParcelType + " not found. Please try again.";         
          var featTargetLyr = typeNamesEnum.FirstOrDefault();
          var pRec = myParcelFabricLayer.GetActiveRecord();
          if (pRec == null)
            MessageBox.Show("There is no Active Record. Please set the active record and try again.");

          var opMerge2 = new EditOperation()
          {
            Name = "Merge2",
            ProgressMessage = "Merging parcels...",
            ShowModalMessageAfterFailure = true,
            SelectNewFeatures = true,
            SelectModifiedFeatures = false
          };
          opMerge2.Merge(featTargetLyr, featSrcLyr, featSrcLyr.GetSelection().GetObjectIDs());
          if (!opMerge2.Execute())
            return opMerge2.ErrorMessage;
          #endregion
        }
        catch (Exception ex)
        {
          return ex.Message;  
        }
        return "";
      });
      if (!string.IsNullOrEmpty(errorMessage))
        MessageBox.Show(errorMessage, "Merge Parcels");
      //When active record is set, and merging into the same parcel type, the original parcels are set historic with RetiredByRecord GUID field
      //When active record is set, the new merged parcel will be tagged with the Active record.
    }
  }
}
